%% Chapter 8: Portfolio
% Example2: Asset Allocation Case Study
%%% This example shows how to set up a basic asset allocation problem that uses mean-variance portfolio optimization to estimate efficient portfolios.
clc
clear
close all
%% Step 1. Defining the portfolio problem.
Asset = { 'Bonds', 'Large-Cap Equities', 'Small-Cap Equities', 'Emerging Equities' };
Price = [ 52.4; 122.7; 35.2; 46.9 ];
Holding = [ 42938; 24449; 42612; 15991 ];
UnitCost = [ 0.001; 0.001; 0.001; 0.004 ];
Blotter = dataset({Price, 'Price'}, {Holding, 'InitHolding'},'obsnames',Asset);
Wealth = sum(Blotter.Price .* Blotter.InitHolding);
Blotter.InitPort = (1/Wealth)*(Blotter.Price .* Blotter.InitHolding);
Blotter.UnitCost = UnitCost;
disp(Blotter);
%% Step 2. Simulating asset prices
AssetMean = [ 0.05; 0.1; 0.12; 0.18 ];
AssetCovar = [ 0.0064 0.00408 0.00192 0;
            0.00408 0.0289 0.0204 0.0119;
            0.00192 0.0204 0.0576 0.0336;
            0 0.0119 0.0336 0.1225 ];
X = portsim(AssetMean'/12, AssetCovar/12, 60); % monthly total returns for 5 years 
[Y, T] = ret2tick(X, [], 1/12); % form total return prices
plot(T, log(Y));
title('Simulated Asset Class Total Return Prices');
xlabel('Year');
ylabel('Log Total Return Price');
legend(Asset,'Location','best');
%% Step 3. Setting up the Portfolio object
p = Portfolio('Name', 'Asset Allocation Portfolio', ...
'AssetList', Asset, 'InitPort', Blotter.InitPort);
p = setDefaultConstraints(p);
p = setGroups(p, [ 0, 1, 1, 1 ], [], 0.85);
p = addGroups(p, [ 0, 0, 0, 1 ], [], 0.35);
%p = setAssetMoments(p, AssetMean/12, AssetCovar/12);
p = estimateAssetMoments(p, Y, 'DataFormat', 'Prices');
p.AssetMean = 12*p.AssetMean;
p.AssetCovar = 12*p.AssetCovar;
%% Step 4. Validate the portfolio problem
[lb, ub] = estimateBounds(p);
display([lb, ub]);
%% Step 5. Plotting the efficient frontier
figure;
plotFrontier(p, 40);
%% Step 6. Evaluating gross vs. net portfolio returns
q = setCosts(p, UnitCost, UnitCost);
%% Step 7. Analyzing descriptive properties of the Portfolio structures
[prsk0, pret0] = estimatePortMoments(p, p.InitPort);
pret = estimatePortReturn(p, p.estimateFrontierLimits);
qret = estimatePortReturn(q, q.estimateFrontierLimits);
fprintf('Annualized Portfolio Returns ...\n');
fprintf(' %6s %6s\n','Gross','Net');
fprintf('Initial Portfolio Return %6.2f %% %6.2f %%\n',100*pret0,100*pret0);
fprintf('Maximum Efficient Portfolio Return %6.2f %% %6.2f %%\n',100*pret(2), 100*qret(2));
%% Step 8. Obtaining a Portfolio at the specified return level on the efficient frontier
Level = 0.3;
qret = estimatePortReturn(q, q.estimateFrontierLimits);
qwgt = estimateFrontierByReturn(q, interp1([0, 1], qret, Level));
[qrsk, qret] = estimatePortMoments(q, qwgt);
fprintf('Portfolio at %g%% return level on efficient frontier ...\n',100*Level);
fprintf('%10s %10s\n','Return','Risk');
fprintf('%10.2f %10.2f\n',100*qret,100*qrsk);
display(qwgt);
%% Step 9. Obtaining a Portfolio at the specified risk levels on the efficient frontier
TargetRisk = [ 0.10; 0.15; 0.20 ];
qwgt2 = estimateFrontierByRisk(q, TargetRisk);
display(qwgt2);
display(estimatePortRisk(q, qwgt2));
[qwgt2, qbuy, qsell] = estimateFrontierByRisk(q, 0.15);
disp(sum(qbuy + qsell)/2)
q = setTurnover(q, 0.15);
[qwgt2, qbuy, qsell] = estimateFrontierByRisk(q, 0.15);
qbuy(abs(qbuy) < 1.0e-5) = 0;
qsell(abs(qsell) < 1.0e-5) = 0; % zero out near 0 trade weights
Blotter.Port = qwgt2;
Blotter.Buy = qbuy;
Blotter.Sell = qsell;
display(Blotter);
TotalCost = Wealth * sum(Blotter.UnitCost .* (Blotter.Buy + Blotter.Sell))
Blotter.Holding = Wealth * (Blotter.Port ./ Blotter.Price);
Blotter.BuyShare = Wealth * (Blotter.Buy ./ Blotter.Price);
Blotter.SellShare = Wealth * (Blotter.Sell ./ Blotter.Price);
Blotter.Buy = [];
Blotter.Sell = [];
Blotter.UnitCost = [];
%% Step 10. Displaying the final results
display(Blotter);
figure;
plotFrontier(q, 40);
hold on
scatter(estimatePortRisk(q, qwgt2), estimatePortReturn(q, qwgt2), 'filled', 'r');
h = legend('Initial Portfolio', 'Efficient Frontier', 'Final Portfolio', 'location', 'best');
set(h, 'Fontsize', 8);
hold off

